/***************************************************************************
                          knetworkconf.h  -  description
                             -------------------
    begin                : Sun Jan 12 00:54:19 UTC 2003
    copyright            : (C) 2003 by Juan Luis Baptiste
    email                : jbaptiste@merlinux.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNETWORKCONF_H
#define KNETWORKCONF_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define PATH_TO_RH_RELOAD_SCRIPT "/etc/init.d/network"

#define DEVICE_UP 0
#define DEVICE_DOWN 1
#define IFACE_UPDOWN_CMD "/usr/share/setup-tool-backends/scripts/network-conf -d enable_iface::"

#include <unistd.h>
#include <sys/types.h>

#include <kapp.h>
#include <qwidget.h>
#include <qpixmap.h>
#include <qprocess.h>
#include <qptrlist.h>
#include <qstringlist.h>
#include <qstring.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <kpushbutton.h>
#include <klistview.h>
#include <klineedit.h>
#include <klistbox.h>
#include <kcombobox.h>
#include <kmessagebox.h>
#include <kaboutapplication.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include "knetworkconfdlg.h"
#include "kadddnsserverdlg.h"
#include "kadddevicedlg.h"
#include "knetworkinfo.h"
#include "kroutinginfo.h"
#include "knetworkconfigparser.h"
#include "kdnsinfo.h"
#include "kaddknownhostdlg.h"
#include "kknownhostinfo.h"
#include "knetworkinterface.h"


/** KNetworkConf is the base class of the project */
class KNetworkConf : public KNetworkConfDlg
{
  Q_OBJECT 
  public:
    /** construtor */
    KNetworkConf(QWidget* parent=0, const char *name=0);
    /** destructor */
    ~KNetworkConf();
    /** Puts the application in read-only mode. This happens when the user runing the application isn't root. */
    void setReadOnly(bool state);    
    void setVersion(QString ver);
    QString getVersion();   
    /** Disables all buttons a line edit widgets when the user has read only access. */
    void disableAll();

    void loadNetworkDevicesInfo();
    void loadRoutingAndDNSInfo();
    void loadDNSInfoTab();

  private: // Private attributes
    /**  */
    KNetworkConfigParser *config;
    KNetworkInterface * getDeviceInfo(QString device);
    QString getDeviceName(QString ipAddr);
  /** Creates a QStringList with the IP addresses contained in the QListBox of name servers. */
    QStringList getNamserversList(KListBox * serverList);
/** Creates a QPtrList<KKownHostInfo> with the info contained in the KListView of name servers. */    
    QPtrList<KKnownHostInfo> getKnownHostsList(KListView * hostsList);    
    QString currentDevice;
    KRoutingInfo *routingInfo;
    KDNSInfo *dnsInfo;
    bool reloaded;    
    QString commandOutput;
    QProcess *procUpdateDevice;
    QProcess *procDeviceState;    
    QStringList deviceNamesList;
    bool devicesModified;
    QPtrList<KNetworkInterface> deviceList;
    bool readOnly;
    QPtrList<KKnownHostInfo> knownHostsList;
    bool nameServersModified;
    /** The program's version. */
    QString version;
    bool modified;
    bool devStateChanged;
  /**  */
  KNetworkInfo * netInfo;
  /** Has the errors throwed by GST when executed. */
  QString commandErrOutput;
    /** Changes the state of device 'dev' to DEVICE_UP or DEVICE_DOWN.
    Return true on success, false on failure.  */
    void changeDeviceState(QString dev, int state);
  public slots:
      virtual void saveInfoSlot();

  private slots:
    void updateDevicesState();  
    /** Enables the configure and remove buttons. */
    virtual void enableButtonsSlot();
    /** opens the add server dialog. */
    virtual void addServerSlot();
    /** Terminates the application*/
    virtual void quitSlot();
    virtual void readFromStdout();
    virtual void readFromStdoutUpDown();
    virtual void upDownInterfaceSlot();
   /** Pops up the window for adding a new interface. */
    virtual void configureDeviceSlot();    
    void enableApplyButtonSlot();  

    /** Shows the help browser. Hopefully some day it will be one :-). */
    virtual void helpSlot();
    virtual void enableApplyButtonSlot(bool);
    virtual void enableApplyButtonSlot(const QString &text);
    /** Saves all the modified info of devices, routes,etc. */
    virtual void moveDownServerSlot();
    virtual void moveUpServerSlot();
    virtual void removeServerSlot();
    bool valuesChanged(KNetworkInterface *dev,
                        QString bootProto,
                        QString netmask,
                        QString ipAddr,
                        bool onBoot,
                        QString desc);
    /** Returns a list of strings of all the configured devices. */
    QStringList getDeviceList();
    virtual void updateWidgets();
  /** Sets the QPushButton::autoResize() in true for all buttons. */
  void makeButtonsResizeable();

    void paintListView();
    void updateNetworkDevicesInfo();
    void updateDhcpDevicesInfo();
    void changeState();
  /** Adds a new host to the KListView that has the known hosts. */
    void addKnownHostSlot();
    void aboutSlot();
  /** Edits the info about a known host. */
  virtual void editKnownHostSlot();
  /** Removes a known host from the list view */
  virtual void removeKnownHostSlot();
  /** No descriptions */
  void readFromStdErrUpDown();
  void loadNetworkInfo();
  /** Shows the main window after the network info has been loaded. */
  void showMainWindow();
  void verifyDeviceStateChanged();
  /** Tries to start the network. */
  virtual void startNetworkSlot();
  /** Tries to stop the network. */
  virtual void stopNetworkSlot();
  /** Sees if a device is active or not in the ifconfig output. Not very nice, but it works. Inthe future, this has to be managed by gst. */
  bool isDeviceActive(QString device, QString ifconfigOutput);
    /** No descriptions */
  void getNewNetworkInfo();
  /** No descriptions */
  void loadAgainNetworkInfoSlot();
  
};

#endif
